@@ -47,7 +47,7 @@ for i, art in enumerate(_articles):
 re_nameImdbIndex = re.compile(r'\(([IVXLCDM]+)\)')
 
 HELP = """imdbpy2sql.py usage:
-    %s -d /directory/with/PlainTextDataFiles/ -u URI
+    %s -d /directory/with/PlainTextDataFiles/ -u URI [--COMPATIBILITY-OPTIONS]
 
         # NOTE: URI is something along the line:
                 scheme://[user[:password]@]host[:port]/database[?parameters]
@@ -58,6 +58,14 @@ HELP = """imdbpy2sql.py usage:
                 sqlite:/tmp/imdb.db
                 sqlite:/C|/full/path/to/database
 
+        # NOTE: --COMPATIBILITY-OPTIONS can be one of:
+            --mysql-innodb          insert data into a MySQL MyISAM db,
+                                    and then convert it to InnoDB.
+            --ms-sqlserver          compatibility mode for Microsoft SQL Server
+                                    and SQL Express.
+            --sqlite-transactions   uses transactions, to speed-up SQLite.
+
+
                 See README.sqldb for more information.
 """ % sys.argv[0]
 
@@ -70,13 +78,35 @@ CUSTOM_QUERIES = {}
 # Allowed time specification, for custom queries.
 ALLOWED_TIMES = ('BEGIN', 'BEFORE_DROP', 'BEFORE_CREATE', 'AFTER_CREATE',
                 'BEFORE_MOVIES', 'BEFORE_CAST', 'BEFORE_RESTORE',
-                'BEFORE_INDEXES', 'END')
+                'BEFORE_INDEXES', 'END', 'BEFORE_MOVIES_TODB',
+                'AFTER_MOVIES_TODB', 'BEFORE_PERSONS_TODB',
+                'AFTER_PERSONS_TODB','BEFORE_SQLDATA_TODB',
+                'AFTER_SQLDATA_TODB', 'BEFORE_AKAMOVIES_TODB',
+                'AFTER_AKAMOVIES_TODB', 'BEFORE_EVERY_TODB',
+                'AFTER_EVERY_TODB')
 
+# Shortcuts for some compatibility options.
+MYSQLINNODB_OPTS = ['-e',
+        'AFTER_CREATE:FOR_EVERY_TABLE:ALTER TABLE %(table)s ENGINE=MyISAM;',
+        '-e', 'END:FOR_EVERY_TABLE:ALTER TABLE %(table)s ENGINE=InnoDB;']
+SQLSERVER_OPTS = ['-e', 'BEFORE_EVERY_TODB:SET IDENTITY_INSERT %(table)s ON;',
+        '-e', 'AFTER_EVERY_TODB:SET IDENTITY_INSERT %(table)s OFF;']
+SQLITE_OPTS = ['-e', 'BEFORE_EVERY_TODB:BEGIN TRANSACTION;',
+        '-e', 'AFTER_EVERY_TODB:COMMIT;']
+
+if '--mysql-innodb' in sys.argv[1:]:
+    sys.argv += MYSQLINNODB_OPTS
+if '--ms-sqlserver' in sys.argv[1:]:
+    sys.argv += SQLSERVER_OPTS
+if '--sqlite-transactions' in sys.argv[1:]:
+    sys.argv += SQLITE_OPTS
 
 # Manage arguments list.
 try:
     optlist, args = getopt.getopt(sys.argv[1:], 'u:d:e:h',
-                                                ['uri=', 'data=', 'execute='
+                                                ['uri=', 'data=', 'execute=',
+                                                'mysql-innodb', 'ms-sqlserver',
+                                                'sqlite-transactions',
                                                 'help'])
 except getopt.error, e:
     print 'Troubles with arguments.'
@@ -96,7 +126,16 @@ for opt in optlist:
         if when not in ALLOWED_TIMES:
             print 'WARNING: unknown time: "%s"' % when
             continue
-        CUSTOM_QUERIES.setdefault(when, []).append(cmd)
+        if when == 'BEFORE_EVERY_TODB':
+            for nw in ('BEFORE_MOVIES_TODB', 'BEFORE_PERSONS_TODB',
+                        'BEFORE_SQLDATA_TODB', 'BEFORE_AKAMOVIES_TODB'):
+                CUSTOM_QUERIES.setdefault(nw, []).append(cmd)
+        elif when == 'AFTER_EVERY_TODB':
+            for nw in ('AFTER_MOVIES_TODB', 'AFTER_PERSONS_TODB',
+                        'AFTER_SQLDATA_TODB', 'AFTER_AKAMOVIES_TODB'):
+                CUSTOM_QUERIES.setdefault(nw, []).append(cmd)
+        else:
+            CUSTOM_QUERIES.setdefault(when, []).append(cmd)
     elif opt[0] in ('-h', '--help'):
         print HELP
         sys.exit(0)
@@ -233,7 +272,7 @@ def title_soundex(title):
         title = ', '.join(ts[:-1])
     return soundex(title)
 
-def name_soundexes(name):
+def name_soundexes(name, character=False):
     """Return three soundex codes for the given name; the name is assumed
     to be in the 'surname, name' format, without the imdbIndex indication,
     as the one in the analyze_name(name)['name'] value.
@@ -250,8 +289,11 @@ def name_soundexes(name):
     name_normal = normalizeName(name)
     s2 = soundex(name_normal)
     if s1 == s2: s2 = None
-    namesplit = name.split(', ')
-    s3 = soundex(namesplit[0])
+    if not character:
+        namesplit = name.split(', ')
+        s3 = soundex(namesplit[0])
+    else:
+        s3 = soundex(name.split(' ')[-1])
     if s3 and s3 in (s1, s2): s3 = None
     return (s1, s2, s3)
 
@@ -425,6 +467,8 @@ class _BaseCache(dict):
         self._flushing = 0
         self._deferredData = {}
         self._recursionLevel = 0
+        self._table_name = ''
+        self._id_for_custom_q = ''
         if d is not None:
             for k, v in d.iteritems(): self[k] = v
 
@@ -450,10 +494,23 @@ class _BaseCache(dict):
             self._tmpDict.clear()
             return
         if self._tmpDict:
+            # Horrible hack to know if AFTER_%s_TODB has run.
+            _after_has_run = False
+            keys = {'table': self._table_name}
             try:
+                executeCustomQueries('BEFORE_%s_TODB' % self._id_for_custom_q,
+                                    _keys=keys, _timeit=False)
                 self._toDB(quiet)
+                executeCustomQueries('AFTER_%s_TODB' % self._id_for_custom_q,
+                                    _keys=keys, _timeit=False)
+                _after_has_run = True
                 self._tmpDict.clear()
             except OperationalError, e:
+                # XXX: I'm not sure this is the right thing (and way)
+                #      to proceed.
+                if not _after_has_run:
+                    executeCustomQueries('AFTER_%s_TODB'%self._id_for_custom_q,
+                                        _keys=keys, _timeit=False)
                 # Dataset too large; split it in two and retry.
                 # XXX: new code!
                 # the same class instance (self) is used, instead of
@@ -528,6 +585,8 @@ class MoviesCache(_BaseCache):
     def __init__(self, *args, **kwds):
         _BaseCache.__init__(self, *args, **kwds)
         self.movieYear = {}
+        self._table_name = tableName(Title)
+        self._id_for_custom_q = 'MOVIES'
         self.sqlstr, self.converter = createSQLstr(Title, ('id', 'title',
                                     'imdbIndex', 'kindID', 'productionYear',
                                     'phoneticCode', 'episodeOfID',
@@ -619,6 +678,8 @@ class PersonsCache(_BaseCache):
 
     def __init__(self, *args, **kwds):
         _BaseCache.__init__(self, *args, **kwds)
+        self._table_name = tableName(Name)
+        self._id_for_custom_q = 'PERSONS'
         self.sqlstr, self.converter = createSQLstr(Name, ['id', 'name',
                                 'imdbIndex', 'namePcodeCf', 'namePcodeNf',
                                 'surnamePcode'])
@@ -663,6 +724,58 @@ class PersonsCache(_BaseCache):
         CURS.executemany(self.sqlstr, self.converter(l))
 
 
+class CharactersCache(_BaseCache):
+    """Manage the characters list."""
+    counter = counter()
+
+    def __init__(self, *args, **kwds):
+        _BaseCache.__init__(self, *args, **kwds)
+        self._table_name = tableName(CharName)
+        self._id_for_custom_q = 'CHARACTERS'
+        self.sqlstr, self.converter = createSQLstr(CharName, ['id', 'name',
+                                'imdbIndex', 'namePcodeNf', 'surnamePcode'])
+
+    def populate(self):
+        print ' * POPULATING CharactersCache...'
+        nameTbl = tableName(CharName)
+        personidCol = colName(CharName, 'id')
+        nameCol = colName(CharName, 'name')
+        imdbindexCol = colName(CharName, 'imdbIndex')
+        CURS.execute('SELECT %s, %s, %s FROM %s;' % (personidCol, nameCol,
+                                                    imdbindexCol, nameTbl))
+        _oldcacheValues = CharName.sqlmeta.cacheValues
+        CharName.sqlmeta.cacheValues = False
+        for x in fetchsome(CURS, self.flushEvery):
+            nd = {'name': unicode(x[1], 'utf_8')}
+            if x[2]: nd['imdbIndex'] = x[2]
+            name = build_name(nd, canonical=1)
+            dict.__setitem__(self, name, x[0])
+        self.counter = counter(CharName.select().count() + 1)
+        CharName.sqlmeta.cacheValues = _oldcacheValues
+
+    def _toDB(self, quiet=0):
+        if not quiet:
+            print ' * FLUSHING CharactersCache...'
+            sys.stdout.flush()
+        l = []
+        lapp = l.append
+        tmpDictiter = self._tmpDict.iteritems
+        for k, v in tmpDictiter():
+            try:
+                t = analyze_name(k)
+            except IMDbParserError:
+                if k and k.strip():
+                    print 'WARNING CharactersCache._toDB() invalid name:', _(k)
+                continue
+            tget = t.get
+            name = tget('name')
+            namePcodeCf, namePcodeNf, surnamePcode = name_soundexes(name,
+                                                                character=True)
+            lapp((v, name, tget('imdbIndex'),
+                namePcodeCf, surnamePcode))
+        CURS.executemany(self.sqlstr, self.converter(l))
+
+
 class SQLData(dict):
     """Variable set of information, to be stored from time to time
     to the SQL database."""
@@ -681,6 +794,7 @@ class SQLData(dict):
         self.sqlString = sqlString
         self.converter = converter
         self._recursionLevel = 1
+        self._table_name = tableName(table)
         for k, v in d.items(): self[k] = v
 
     def __setitem__(self, key, value):
@@ -709,11 +823,21 @@ class SQLData(dict):
             self.clear()
             self.counter = self.counterInit
             return
+        keys = {'table': self._table_name}
+        _after_has_run = False
         try:
+            executeCustomQueries('BEFORE_SQLDATA_TODB', _keys=keys,
+                                _timeit=False)
             self._toDB()
+            executeCustomQueries('AFTER_SQLDATA_TODB', _keys=keys,
+                                _timeit=False)
+            _after_has_run = True
             self.clear()
             self.counter = self.counterInit
         except OperationalError, e:
+            if not _after_has_run:
+                executeCustomQueries('AFTER_SQLDATA_TODB', _keys=keys,
+                                    _timeit=False)
             print ' * TOO MANY DATA (%s items), SPLITTING (run #%d)...' % \
                     (len(self), self._recursionLevel)
             self._recursionLevel += 1
@@ -825,7 +949,7 @@ def doCast(fp, roleid, rolename):
     name = ''
     roleidVal = RawValue('roleID', roleid)
     sqldata = SQLData(table=CastInfo, cols=['personID', 'movieID',
-                        'personRole', 'note', 'nrOrder', roleidVal])
+                        'personRoleID', 'note', 'nrOrder', roleidVal])
     if rolename == 'miscellaneous crew': sqldata.flushEvery = 10000
     for line in fp:
         if line and line[0] != '\t':
@@ -872,7 +996,13 @@ def doCast(fp, roleid, rolename):
                         except ValueError:
                             pass
         movieid = CACHE_MID.addUnique(title)
-        sqldata.add((pid, movieid, role, note, order))
+        if role is not None:
+            roles = role.split('/')
+            for role in roles:
+                cid = CACHE_CID.addUnique(role)
+                sqldata.add((pid, movieid, cid, note, order))
+        else:
+            sqldata.add((pid, movieid, None, note, order))
         if count % 10000 == 0:
             print 'SCANNING %s:' % rolename,
             print _(name)
@@ -896,9 +1026,15 @@ def castLists():
         try:
             f = SourceFile(fname, start=CAST_START, stop=CAST_STOP)
         except IOError:
+            if rolename == 'actress':
+                CACHE_CID.flush()
+                CACHE_CID.clear()
             continue
         doCast(f, roleid, rolename)
         f.close()
+        if rolename == 'actress':
+            CACHE_CID.flush()
+            CACHE_CID.clear()
         t('castLists(%s)' % rolename)
 
 
@@ -944,6 +1080,8 @@ class AkasMoviesCache(MoviesCache):
         self.flushEvery = 50000
         self.notes = {}
         self.ids = {}
+        self._table_name = tableName(AkaTitle)
+        self._id_for_custom_q = 'AKAMOVIES'
         self.sqlstr, self.converter = createSQLstr(AkaTitle, ('id', 'movieID',
                             'title', 'imdbIndex', 'kindID', 'productionYear',
                             'phoneticCode', 'episodeOfID', 'seasonNr',
@@ -1191,7 +1329,7 @@ def nmmvFiles(fp, funct, fname):
         guestid = RoleType.select(RoleType.q.role == 'guest')[0].id
         roleid = str(guestid)
         guestdata = SQLData(table=CastInfo,
-                cols=['personID', 'movieID', 'personRole', 'note',
+                cols=['personID', 'movieID', 'personRoleID', 'note',
                 RawValue('roleID', roleid)], flushEvery=10000)
         akanamesdata = SQLData(table=AkaName, cols=['personID', 'name',
                 'imdbIndex', 'namePcodeCf', 'namePcodeNf', 'surnamePcode'])
@@ -1234,7 +1372,15 @@ def nmmvFiles(fp, funct, fname):
                         title = i.get('long imdb canonical title')
                         if not title: continue
                         movieid = CACHE_MID.addUnique(title)
-                        guestdata.add((mopid, movieid, i.currentRole or None,
+                        crole = i.currentRole
+                        if isinstance(crole, list):
+                            crole = ' / '.join([x.get('long imdb name', u'')
+                                                for x in crole])
+                        if not crole:
+                            crole = None
+                        else:
+                            crole = unicode(crole).encode('utf_8')
+                        guestdata.add((mopid, movieid, crole,
                                         i.notes or None))
                         continue
                     if k in ('plot', 'mini biography'):
@@ -1389,7 +1535,7 @@ def getTopBottomRating():
             mid = CACHE_MID.addUnique(title)
             if what == 'top 250 rank': rank = count
             else: rank = 11 - count
-            sqldata.add((mid, rank))
+            sqldata.add((mid, str(rank)))
             count += 1
         sqldata.flush()
         fp.close()
@@ -1446,6 +1592,7 @@ def completeCast():
 # global instances
 CACHE_MID = MoviesCache()
 CACHE_PID = PersonsCache()
+CACHE_CID = CharactersCache()
 
 
 def _cmpfunc(x, y):
@@ -1556,8 +1703,9 @@ def _executeQuery(query):
         print 'FAILED (%s)!' % e
 
 
-def executeCustomQueries(when):
+def executeCustomQueries(when, _keys=None, _timeit=True):
     """Run custom queries as specified on the command line."""
+    if _keys is None: _keys = {}
     for query in CUSTOM_QUERIES.get(when, []):
         print 'EXECUTING "%s:%s"...' % (when, query)
         sys.stdout.flush()
@@ -1567,13 +1715,17 @@ def executeCustomQueries(when):
             tables = [x[0] for x in CURS.fetchall()]
             for table in tables:
                 try:
-                    _executeQuery(query % {'table': table})
-                    t('%s command' % when)
+                    keys = {'table': table}
+                    keys.update(_keys)
+                    _executeQuery(query % keys)
+                    if _timeit:
+                        t('%s command' % when)
                 except Exception, e:
                     print 'FAILED (%s)!' % e
         else:
-            _executeQuery(query)
-            t('%s command' % when)
+            _executeQuery(query % _keys)
+            if _timeit:
+                t('%s command' % when)
 
 
 # begin the iterations...
@@ -1685,6 +1837,8 @@ def run():
     CACHE_MID.clear()
     CACHE_PID.flush()
     CACHE_PID.clear()
+    CACHE_CID.flush()
+    CACHE_CID.clear()
     t('fushing caches...')
 
     print 'TOTAL TIME TO INSERT DATA: %d minutes, %d seconds' % \
@@ -1718,6 +1872,8 @@ def _kdb_handler(signum, frame):
     except IntegrityError: pass
     try: CACHE_PID.flush()
     except IntegrityError: pass
+    try: CACHE_CID.flush()
+    except IntegrityError: pass
     print 'DONE! (in %d minutes, %d seconds)' % \
             divmod(int(time.time())-BEGIN_TIME, 60)
     sys.exit()
