@@ -34,10 +34,11 @@ from sqlobject.sqlbuilder import *
 from dbschema import *
 
 from imdb.parser.common.locsql import IMDbLocalAndSqlAccessSystem, \
-                    scan_names, scan_titles, titleVariations, nameVariations
-from imdb.utils import canonicalTitle, canonicalName, normalizeTitle, \
-                        normalizeName, build_title, build_name, \
-                        analyze_name, analyze_title, re_episodes, _articles
+                    scan_names, scan_titles, titleVariations, \
+                    nameVariations, merge_roles
+from imdb.utils import normalizeTitle, normalizeName, build_title, \
+                        build_name, analyze_name, analyze_title, \
+                        re_episodes, _articles
 from imdb.Person import Person
 from imdb.Movie import Movie
 from imdb._exceptions import IMDbDataAccessError, IMDbError
@@ -286,6 +287,13 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
             raise IMDbError, 'personID "%s" can\'t be converted to integer' % \
                             personID
 
+    def _normalize_characterID(self, characterID):
+        """Normalize the given characterID."""
+        try:
+            return int(characterID)
+        except (ValueError, OverflowError):
+            raise IMDbError, 'characterID "%s" can\'t be converted to integer' \
+                            % characterID
 
     def get_imdbMovieID(self, movieID):
         """Translate a movieID in an imdbID.
@@ -326,6 +334,23 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
             except: pass
         return imdbID
 
+    def get_imdbCharacterID(self, characterID):
+        """Translate a characterID in an imdbID.
+        If not in the database, try an Exact Primary Name search on IMDb;
+        return None if it's unable to get the imdbID.
+        """
+        try: character = CharName.get(characterID)
+        except SQLObjectNotFound: return None
+        imdbID = character.imdbID
+        if imdbID is not None: return '%07d' % imdbID
+        n_dict = {'name': character.name, 'imdbIndex': character.imdbIndex}
+        namline = build_name(n_dict, canonical=1)
+        imdbID = self.character2imdbID(namline)
+        if imdbID is not None:
+            try: character.imdbID = int(imdbID)
+            except: pass
+        return imdbID
+
     def do_adult_search(self, doAdult):
         """If set to 0 or False, movies in the Adult category are not
         episodeOf = title_dict.get('episode of')
@@ -431,7 +456,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         if not res:
             raise IMDbDataAccessError, 'unable to get movieID "%s"' % movieID
         # Collect cast information.
-        castdata = [[cd.personID, cd.personRole, cd.note, cd.nrOrder,
+        castdata = [[cd.personID, cd.personRoleID, cd.note, cd.nrOrder,
                     self._role[cd.roleID]]
                     for cd in CastInfo.select(CastInfo.q.movieID == movieID)]
         for p in castdata:
@@ -444,12 +469,22 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         for group in castdata:
             duty = group[0][4]
             for pdata in group:
+                curRole = pdata[1]
+                curRoleID = None
+                if curRole is not None:
+                    robj = CharName.get(curRole)
+                    curRole = robj.name
+                    curRoleID = robj.id
                 p = Person(personID=pdata[0], name=pdata[5],
-                            currentRole=pdata[1] or u'', notes=pdata[2] or u'',
+                            currentRole=curRole or u'',
+                            roleID=curRoleID,
+                            notes=pdata[2] or u'',
                             accessSystem='sql')
                 if pdata[6]: p['imdbIndex'] = pdata[6]
                 p.billingPos = pdata[3]
                 res.setdefault(duty, []).append(p)
+            if duty == 'cast':
+                res[duty] = merge_roles(res[duty])
             res[duty].sort()
         # Info about the movie.
         minfo = [(self._info[m.infoTypeID], m.info, m.note)
@@ -612,10 +647,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
             raise IMDbDataAccessError, \
                     'unable to search the database: "%s"' % str(e)
 
-        resultsST = results
-        if not self.doAdult: resultsST = 0
-        res = scan_names(qr, name1, name2, name3, resultsST)
-        if results > 0: res[:] = res[:results]
+        res = scan_names(qr, name1, name2, name3, results)
         res[:] = [x[1] for x in res]
         # Purge empty imdbIndex.
         returnl = []
@@ -639,7 +671,8 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         if not res:
             raise IMDbDataAccessError, 'unable to get personID "%s"' % personID
         # Collect cast information.
-        castdata = [(cd.movieID, cd.personRole, cd.note, self._role[cd.roleID],
+        castdata = [(cd.movieID, cd.personRoleID, cd.note,
+                    self._role[cd.roleID],
                     get_movie_data(cd.movieID, self._kind))
                 for cd in CastInfo.select(CastInfo.q.personID == personID)]
         # Regroup by role/duty (cast, writer, director, ...)
@@ -656,8 +689,15 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
                     if orig_duty not in ('actor', 'actress'):
                         if note: note = ' %s' % note
                         note = '[%s]%s' % (orig_duty, note)
+                curRole = mdata[1]
+                curRoleID = None
+                if curRole is not None:
+                    robj = CharName.get(curRole)
+                    curRole = robj.name
+                    curRoleID = robj.id
                 m = Movie(movieID=mdata[0], data=mdata[4],
-                            currentRole=mdata[1] or u'',
+                            currentRole=curRole or u'',
+                            roleID=curRoleID,
                             notes=note, accessSystem='sql')
                 if duty != 'episodes':
                     res.setdefault(duty, []).append(m)
@@ -670,6 +710,9 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
             res['episodes'] = episodes
         for duty in seenDuties:
             if res.has_key(duty):
+                if duty in ('actor', 'actress', 'himself', 'herself',
+                            'themselves'):
+                    res[duty] = merge_roles(res[duty])
                 res[duty].sort()
         # XXX: is 'guest' still needed?  I think every GA reference in
         #      the biographies.list file was removed.
@@ -732,6 +775,101 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
     get_person_other_works = get_person_main
     get_person_episodes = get_person_main
 
+    def _search_character(self, name, results):
+        name = name.strip()
+        if not name: return []
+        s_name = analyze_name(name)['name']
+        if not s_name: return []
+        if isinstance(s_name, UnicodeType):
+            s_name = s_name.encode('ascii', 'ignore')
+        s_name = normalizeName(s_name)
+        soundexCode = soundex(s_name)
+        surname = s_name.split(' ')[-1]
+        surnameSoundex = soundex(surname)
+        name2 = ''
+        soundexName2 = None
+        nsplit = s_name.split()
+        if len(nsplit) > 1:
+            name2 = '%s %s' % (nsplit[-1], ' '.join(nsplit[:-1]))
+            if s_name == name2:
+                name2 = ''
+            else:
+                soundexName2 = soundex(name2)
+        # If the soundex is None, compare only with the first
+        # phoneticCode column.
+        if soundexCode is not None:
+            if soundexName2 is not None:
+                condition = OR(surnameSoundex == CharName.q.surnamePcode,
+                            IN(CharName.q.namePcodeNf, [soundexCode,
+                                                        soundexName2]),
+                            IN(CharName.q.surnamePcode, [soundexCode,
+                                                        soundexName2]))
+            else:
+                condition = OR(surnameSoundex == CharName.q.surnamePcode,
+                            IN(soundexCode, [CharName.q.namePcodeNf,
+                                            CharName.q.surnamePcode]))
+        else:
+            condition = ISNULL(Name.q.namePcodeNf)
+        try:
+            qr = [(q.id, {'name': q.name, 'imdbIndex': q.imdbIndex})
+                    for q in CharName.select(condition)]
+        except SQLObjectNotFound, e:
+            raise IMDbDataAccessError, \
+                    'unable to search the database: "%s"' % str(e)
+        res = scan_names(qr, s_name, name2, '', results,
+                        _scan_character=True)
+        res[:] = [x[1] for x in res]
+        # Purge empty imdbIndex.
+        returnl = []
+        for x in res:
+            tmpd = x[1]
+            if tmpd['imdbIndex'] is None:
+                del tmpd['imdbIndex']
+            returnl.append((x[0], tmpd))
+        return returnl
+
+    def get_character_main(self, characterID, results=1000):
+        # Every person information is retrieved from here.
+        infosets = self.get_character_infoset()
+        try:
+            c = CharName.get(characterID)
+        except SQLObjectNotFound, e:
+            raise IMDbDataAccessError, \
+                    'unable to get characterID "%s": "%s"' % (characterID, e)
+        res = {'name': c.name, 'imdbIndex': c.imdbIndex}
+        if res['imdbIndex'] is None: del res['imdbIndex']
+        if not res:
+            raise IMDbDataAccessError, 'unable to get characterID "%s"' % \
+                                        characterID
+        # Collect filmography information.
+        items = CastInfo.select(CastInfo.q.personRoleID == characterID)
+        if results > 0:
+            items = items[:results]
+        filmodata = [(cd.movieID, cd.personID, cd.note,
+                    get_movie_data(cd.movieID, self._kind)) for cd in items
+                    if self._role[cd.roleID] in ('actor', 'actress')]
+        fdata = []
+        for f in filmodata:
+            curRole = None
+            curRoleID = f[1]
+            note = f[2] or u''
+            if curRoleID is not None:
+                robj = Name.get(curRoleID)
+                curRole = robj.name
+            m = Movie(movieID=f[0], data=f[3],
+                        currentRole=curRole or u'',
+                        roleID=curRoleID, roleIsPerson=True,
+                        notes=note, accessSystem='sql')
+            fdata.append(m)
+        fdata = merge_roles(fdata)
+        fdata.sort()
+        if fdata:
+            res['filmography'] = fdata
+        return {'data': res, 'info sets': infosets}
+
+    get_character_filmography = get_character_main
+    get_character_biography = get_character_main
+
     def __del__(self):
         """Ensure that the connection is closed."""
         if not hasattr(self, '_connection'): return
